#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include <sys\stat.h>
/*
The track ordering is causing problems when using disk images for VMWare.
VMWare expects to have the tracks arranged C0H0, C0H1, C1H0, C1H1, C2H0, C2H1, ... 
Cpmtools doesn't care about cylinders or heads; it just writes
tracks, so a disk image written by cpmtools ends up C0H0, C1H0, C2H0, C3H0, ...

Converting a disk image from VMWare to cpmtools needs option "vmw-to-cpmt"
Converting a disk image from cpmtools to VMWare needs option "cpmt-to-vmw"

Visit my blog at http://www.z80.eu/blog/ for more infos, Peter Dassow 08-2012
*/

#define SECTORSIZE 512
#define SECSPERTRACK 18
#define TRACKS 80

int main(int argc,char *argv[])
{  int fdi,fdt;
   unsigned char buffer[SECTORSIZE*SECSPERTRACK];
   int rd,wr,trks;
   int toggle,split;
   long filepos,offset;

   if (argc!=4) {
     fputs("usage: cpm86cnv input-image-file output-image-file vmw-to-cpmt|cpmt-to-vmw\n",stderr);
     return(1);
   }
   if (strcmp(argv[3],"vmw-to-cpmt")==0) {
     split=1;
   }
   else if (strcmp(argv[3],"cpmt-to-vmw")==0) {
     split=0;
   }
   else {
     fputs("usage: option is only vmw-to-cpmt or cpmt-to-vmw !\n",stderr);
     return(1);
   }
   /* expecting now split to be 1 or 0 for split or reassemble */
   puts("Opening input file...");
   if (split==1) {
    fdi=open(argv[1],O_RDONLY|O_BINARY,S_IREAD);
    if (fdi<0) {
      fputs("error: cpm86cnv could not open input-file\n",stderr);
      return(1);
    }
    fdt=open(argv[2],O_CREAT|O_WRONLY|O_BINARY,S_IREAD|S_IWRITE);
    if (fdt<0) {
      fputs("error: cpm86cnv could not write output file\n",stderr);
      return(1);
    }
    offset=(long)SECTORSIZE*(long)SECSPERTRACK*(long)TRACKS;
    trks=0;
    toggle=0;
    rd=read(fdi,buffer,SECSPERTRACK*SECTORSIZE);
    while ((rd==SECTORSIZE*SECSPERTRACK) &&(trks<TRACKS) && (toggle<2)) {
      if (toggle==0) {
	filepos=lseek(fdt,(long)trks*SECTORSIZE*SECSPERTRACK,SEEK_SET);
	printf("Read Track %d Head %d at %ld (%d bytes)...\n",trks,toggle,filepos,rd);
	wr=write(fdt,buffer,rd);
	if (wr!=SECSPERTRACK*SECTORSIZE) {
	  fprintf(stderr,"error: cpm86cnv could not write further tracks (Track #%d Head 0)\n",trks);
	  toggle=2; /* leaves while loop */
	}
	else
	  toggle=1;
      }
      else {
	filepos=lseek(fdt,offset+(long)trks*SECTORSIZE*SECSPERTRACK,SEEK_SET);
	printf("Read Track %d Head %d at %ld (%d bytes)...\n",trks,toggle,filepos,rd);
	wr=write(fdt,buffer,rd);
	if (wr!=SECSPERTRACK*SECTORSIZE) {
	  fprintf(stderr,"error: cpm86cnv could not write further tracks (Track #%d Head 1)\n",trks);
	  toggle=2; /* leaves while loop */
	}
	else
	  toggle=0;
	trks++;
      }
      /* "trks" can be incremented after reading TWO tracks */
      fflush(stdout);
      rd=read(fdi,buffer,SECSPERTRACK*SECTORSIZE);
    } /* while ends */
    close(fdi);
    close(fdt);
    puts("Output file written.");
   } /* split branch ends here */
   else { /* merge branch starts here */
    fdi=open(argv[1],O_RDONLY|O_BINARY,S_IREAD);
    if (fdi<0) {
      fputs("error: cpm86cnv could not open input-file\n",stderr);
      return(1);
    }
    fdt=open(argv[2],O_CREAT|O_WRONLY|O_BINARY,S_IREAD|S_IWRITE);
    if (fdt<0) {
      fputs("error: cpm86cnv could not write output file\n",stderr);
      return(1);
    }
    /* Head 0 starts at filepos 0, Head 1 starts at filepos SECTORSIZExSECSPERTRACKxTRACKS */

    offset=(long)SECTORSIZE*(long)SECSPERTRACK*(long)TRACKS;
    trks=0;
    toggle=0;
    rd=SECTORSIZE*SECSPERTRACK;

    while ((trks<TRACKS)&&(rd>0)&&(toggle==0)){
      filepos=lseek(fdi,(long)trks*(long)SECTORSIZE*(long)SECSPERTRACK,SEEK_SET);
      rd=read(fdi,buffer,SECTORSIZE*SECSPERTRACK);
      printf("Reading Track %d Head 0 from pos %ld (%d bytes)...\n",trks,filepos,rd);
      if (rd>0) {
	wr=write(fdt,buffer,rd);
	if (wr!=rd) {
	  fputs("error: cpm86cnv could not write track to output file\n",stderr);
	  toggle=1;
	}
      }
      filepos=lseek(fdi,offset+(long)trks*(long)SECTORSIZE*(long)SECSPERTRACK,SEEK_SET);
      rd=read(fdi,buffer,SECTORSIZE*SECSPERTRACK);
      printf("Reading Track %d Head 1 from pos %ld (%d bytes)...\n",trks,filepos,rd);
      if (rd>0) {
	wr=write(fdt,buffer,rd);
	if (wr!=rd) {
	  fputs("error: cpm86cnv could not write track to output file\n",stderr);
	  toggle=1;
	}
      }
      trks++;
    }
    close(fdi);
    close(fdt);
    if (toggle==0) puts("Output file written.");
   } /* merge branchs ends here */
   return(0);
}
